from setuptools import setup

setup(name='pydesy',
      version='0.0.5',
      description='Geodetic library',
      packages=['pydesy'],
      author='Maxim Shmakov',
      author_email='shmakov.maxim2001@yandex.ru',
      install_requires=['numpy', 'matplotlib'],
      url='https://github.com/ChevisOfficial/pydesy',
      zip_safe=False)
